# frozen_string_literal: true

class ComplexSession < ActiveRecord::Base
  belongs_to :complex_subject, foreign_key: 'parent_source_identifier', primary_key: 'source_identifier', optional: true

  has_many :complex_modalities, foreign_key: 'parent_source_identifier', primary_key: 'source_identifier', dependent: :destroy

  attribute :source_identifier, :string, default: -> { SecureRandom.uuid }

  validates :source_identifier, presence: true, uniqueness: true
  validates :parent_source_identifier, presence: true
  validates :session_title, presence: { message: "Session folder name can't be blank" }
  validates :session_title, uniqueness: { scope: :parent_source_identifier,  message: "Session folder name is already taken" }
  validates :session_title, format: { with: /\A[a-zA-Z0-9][a-zA-Z0-9\-\. ]*\z/, message: "Folder name not valid. Folder name should be a combination of only lowercase letters, numbers, hyphen, dot, and space. The name and should start with a letter or number" }

  after_save :save_work_meta_json_file_to_s3, :reindex_crc_dataset
  before_validation :fix_session_title

  delegate :crc_dataset, to: :complex_subject

  def file_sets
    FileSet.where(for_complex_identifier: source_identifier)
  end

  def file_sets_fast_load_metadata
    FileSetFastLoadMetadata.where(work_source_identifier: source_identifier)
  end

  

  private

  def fix_session_title
    self.session_title = self.session_title.downcase.gsub(/^[^a-z]+|[^\w\s.-]/i, '')
  end

  def reindex_crc_dataset
    Hyrax::ReindexCrcDatasetJob.perform_later(crc_dataset.id)
  end

  def save_work_meta_json_file_to_s3
    s3 = S3StorageService.new
    s3.init_client
    bucket_name = s3.sanitise_name(crc_dataset.id)
    s3.create_bucket(bucket_name) unless s3.bucket_exists?(bucket_name)
    s3.add_content(bucket_name, "#{complex_subject.subject_title}/#{session_title}/metadata.json", to_json)
  end
end
